unit RSSD1289;

{
Copyright (C) 2016 - Ronald Daleske


some procedures in this unit have taken some ideas of the C sources of:

Rinky-Dink Electronics, Henning Karlsen
http://www.rinkydinkelectronics.com/library.php


Licence
=======

 LGPLv2.1 with static linking exception (See COPYING.modifiedLGPL.txt)

}


{$mode objfpc}{$H+}

interface

uses
  Threads,
  Platform,
  RGPIO;


const



  // Festwerte fuer Farben
  cl_White    : Word = $FFFF;
  cl_Black    : Word = $0000;
  cl_Red      : Word = $F800;
  cl_Green    : Word = $0400;
  cl_Blue     : Word = $001F;
  cl_Yellow   : Word = $FFE0;
  cl_Purple   : Word = $8010;
  cl_Aqua     : Word = $07FF;
  cl_Gray     : Word = $8410;
  cl_DarkGray : Word = $8410;  // !!! Anpassen
  cl_Silver   : Word = $C618;
  cl_Maroon   : Word = $8000;
  cl_Olive    : Word = $8400;
  cl_Lime     : Word = $07E0;
  cl_Teal     : Word = $0410;
  cl_Navy     : Word = $0010;
  cl_Fuchsia  : Word = $F81F;







procedure clear_Screen(Pixel_Color : Word);
procedure drawPixel_Landscape(Pixel_Color, Pos_x, Pos_y : Word);
procedure drawVertikalLine_Landscape(Pixel_Color, Pos_x, Pos_y1, Pos_y2 : Word);
procedure draw_Picrure(dPicture : Array of Word);


implementation




{
GPIO4  -> SERIAL_DATA
GPIO17 -> SHIFT_CLOCK
GPIO18 -> LATCH_CLOCK

// Data/Command
GPIO22 -> LCD_RS

// Write Low-Aktiv
GPIO23 -> LCD_WR

// Chip-Select Low-Aktiv
GPIO24 -> LCD_CS

// Reset Low-Aktiv
GPIO27 -> LCD_RESET

}




{ 111111111111111111111111111111111111111111111111111 }
{ *************************************************** }
{ *******         PIN-Routins for LCD         ******* }
{ *************************************************** }


// LCD_RS = 1 -> Data
procedure LCD_RS_Data;
begin
  GPIO24_Set;
end;


// LCD_RS = 0 -> Command
procedure LCD_RS_Command;
begin
  GPIO24_Clear;
end;



// Chip-Select Low-Aktiv = 1 -> inactive
procedure LCD_CS_inactive;
begin
  GPIO22_Set;
end;



// Chip-Select Low-Aktiv = 0 -> active
procedure LCD_CS_active;
begin
  GPIO22_Clear;
end;



// Write Low-Aktiv
// GPIO23 -> LCD_WR
procedure LCD_WR_inactive;
begin
  GPIO23_Set;
end;



// Write Low-Aktiv
// GPIO23 -> LCD_WR
procedure LCD_WR_active;
begin
  GPIO23_Clear;
end;



// Reset Low-Aktiv
// GPIO27 -> LCD_RESET
procedure LCD_RESET_inactive;
begin
  GPIO27_Set;
end;



// Reset Low-Aktiv
// GPIO27 -> LCD_RESET
procedure LCD_RESET_active;
begin
  GPIO27_Clear;
end;



// GPIO4  -> SERIAL_DATA
procedure SERIAL_DATA_Clear;
begin
  GPIO4_Clear;
end;


// GPIO4  -> SERIAL_DATA
procedure SERIAL_DATA_Set;
begin
  GPIO4_Set;
end;



// GPIO17 -> SHIFT_CLOCK
procedure SHIFT_CLOCK_Clear;
begin
  GPIO17_Clear;
end;


// GPIO17 -> SHIFT_CLOCK
procedure SHIFT_CLOCK_Set;
begin
  GPIO17_Set;
end;



// GPIO18 -> LATCH_CLOCK
procedure LATCH_CLOCK_Clear;
begin
  GPIO18_Clear;
end;


// GPIO18 -> LATCH_CLOCK
procedure LATCH_CLOCK_Set;
begin
  GPIO18_Set;
end;




procedure Init_Ground;
begin

  // Initializierung
  GPIO_OUT_Init(GPIO4);
  GPIO_OUT_Init(GPIO17);
  GPIO_OUT_Init(GPIO18);

  GPIO_OUT_Init(GPIO22);
  GPIO_OUT_Init(GPIO23);
  GPIO_OUT_Init(GPIO24);
  GPIO_OUT_Init(GPIO27);

  // Ground-State
  SERIAL_DATA_Clear;
  SHIFT_CLOCK_Clear;
  LATCH_CLOCK_Clear;


  // Daten/Kommandos
  // LCD_RS
  LCD_RS_Command;

  // Chip-Select Low-Aktiv
  // LCD_CS
  LCD_CS_inactive;

  // Write Low-Aktiv
  // LCD_WR
  LCD_WR_inactive;

  // Reset Low-Aktiv
  // LCD_RESET
  LCD_RESET_inactive;

end; { procedure Init_Ground }






{ 222222222222222222222222222222222222222222222222222 }
{ *************************************************** }
{ *******    Data-Register-Routins for LCD    ******* }
{ *************************************************** }


procedure Write_Data_Word(WD_Word : Word);
var
  R_Word_Latch : Word;
  R_counter : Byte;
begin

  for R_counter:=1 to 16 do begin

    // SHIFT_CLOCK_Clear;
    GPIO17_Clear;

    R_Word_Latch:=WD_Word and %0000000000000001;

    if R_Word_Latch=0 then
      // SERIAL_DATA_Clear // Data-Bit=0
      GPIO4_Clear
    else
      // SERIAL_DATA_Set; // Data-Bit=1
      GPIO4_Set;

    // Shift IN Data-Bit
    // SHIFT_CLOCK_Set;
    GPIO17_Set;

    WD_Word:=WD_Word shr 1;

  end; { for R_counter:=1 to 16 do }

  LATCH_CLOCK_Set;
  LATCH_CLOCK_Clear;

end; { procedure Write_Data_Word }





procedure Write_Data_Low_Byte(WD_Byte : Byte);
var
  R_Byte_Latch1, R_Byte_Latch2 : Byte;
  R_counter : Byte;
begin

  R_Byte_Latch1:=WD_Byte;

  for R_counter:=1 to 8 do begin

    SHIFT_CLOCK_Clear;

    R_Byte_Latch2:=R_Byte_Latch1 and %00000001;

    if R_Byte_Latch2=0 then
      SERIAL_DATA_Clear // Data-Bit=0
    else
      SERIAL_DATA_Set; // Data-Bit=1

    NanosecondDelay(10);

    // Shift IN Data-Bit
    SHIFT_CLOCK_Set;

    R_Byte_Latch1:=R_Byte_Latch1 shr 1;

  end; { for R_counter:=1 to 8 do }

  SERIAL_DATA_Clear; // Data-Bit=0

  // 8 Byte -> 0
  for R_counter:=1 to 8 do begin

    SHIFT_CLOCK_Clear;

    NanosecondDelay(10);

    // Shift IN Data-Bit
    SHIFT_CLOCK_Set;

    NanosecondDelay(10);

  end; { for R_counter:=1 to 8 do }

  LATCH_CLOCK_Set;
  NanosecondDelay(100);

  SERIAL_DATA_Clear;
  SHIFT_CLOCK_Clear;
  LATCH_CLOCK_Clear;

end; { procedure Write_Data_Low_Byte }




procedure Write_Data_Hight_Low_Byte(Hight_Byte, Low_Byte : Byte);
var
  R_Byte_Latch1, R_Byte_Latch2 : Byte;
  R_counter : Byte;
begin

  R_Byte_Latch1:=Low_Byte;

  for R_counter:=1 to 8 do begin

    SHIFT_CLOCK_Clear;

    R_Byte_Latch2:=R_Byte_Latch1 and %00000001;

    if R_Byte_Latch2=0 then
      SERIAL_DATA_Clear // Data-Bit=0
    else
      SERIAL_DATA_Set; // Data-Bit=1

    NanosecondDelay(10);

    // Shift IN Data-Bit
    SHIFT_CLOCK_Set;

    R_Byte_Latch1:=R_Byte_Latch1 shr 1;

  end; { for R_counter:=1 to 8 do }

  R_Byte_Latch1:=Hight_Byte;

  for R_counter:=1 to 8 do begin

    SHIFT_CLOCK_Clear;

    R_Byte_Latch2:=R_Byte_Latch1 and %00000001;

    if R_Byte_Latch2=0 then
      SERIAL_DATA_Clear // Data-Bit=0
    else
      SERIAL_DATA_Set; // Data-Bit=1

    NanosecondDelay(10);

    // Shift IN Data-Bit
    SHIFT_CLOCK_Set;

    R_Byte_Latch1:=R_Byte_Latch1 shr 1;

  end; { for R_counter:=1 to 8 do }

  LATCH_CLOCK_Set;
  NanosecondDelay(100);

  SERIAL_DATA_Clear;
  SHIFT_CLOCK_Clear;
  LATCH_CLOCK_Clear;

end; { procedure Write_Data_Hight_Low_Byte }




{ 333333333333333333333333333333333333333333333333333 }
{ *************************************************** }
{ *******        Basic-Routins for LCD        ******* }
{ *************************************************** }



procedure LCD_Write_COM(COM_Byte : Byte);
begin

  // Data/Command
  // LCD_RS = 0 -> Command
  LCD_RS_Command;

  NanosecondDelay(10);

  // DB8->DB15 = 0
  // DB0->DB7
  Write_Data_Low_Byte(COM_Byte);

  LCD_WR_active;
  NanosecondDelay(20);
  LCD_WR_inactive;

end; { procedure LCD_Write_COM }




procedure LCD_Write_DATA(DATA_Byte_High, DATA_Byte_Low : Byte);
begin

  LCD_RS_Data;

  Write_Data_Hight_Low_Byte(DATA_Byte_High, DATA_Byte_Low);

  LCD_WR_active;
  NanosecondDelay(20);
  LCD_WR_inactive;

end; { procedure LCD_Write_DATA }






procedure LCD_Write_DATA_Word(DATA_Word : Word);
begin

  LCD_RS_Data;

  Write_Data_Word(DATA_Word);

  LCD_WR_active;
  NanosecondDelay(20);
  LCD_WR_inactive;

end; { procedure LCD_Write_DATA_Word }






procedure LCD_Write_COM_DATA(COM_Byte, DATA_Byte_High, DATA_Byte_Low : Byte);
begin

  LCD_Write_COM(COM_Byte);
  LCD_Write_DATA(DATA_Byte_High, DATA_Byte_Low);

end; { procedure LCD_Write_COM_DATA }





{ *************************************************** }
{ *******       Service-Routins for LCD       ******* }
{ *************************************************** }



procedure Init_SSD1289;
begin

  // send RESET to LCD
  LCD_RESET_inactive;
  MicrosecondDelay(1);
  LCD_RESET_active;
  MillisecondDelay(15);
  LCD_RESET_inactive;

  // Send INIT-Data

  MicrosecondDelay(1);

  // Chip-Select Low-Aktiv
  LCD_CS_active;

  // Einschalten des Oszillators
  LCD_Write_COM_DATA($00,$00,$01);

  //Power Control (1)
  LCD_Write_COM_DATA($03,$A8,$A4);

  //Power Control (2)
  LCD_Write_COM_DATA($0C,$00,$00);

  //Power Control (3)
  LCD_Write_COM_DATA($0D,$08,$0C);

  //Power Control (4)
  LCD_Write_COM_DATA($0E,$2B,$00);

  //Power Control (5)
  LCD_Write_COM_DATA($1E,$00,$B7);

  //Driver Output Control RL=0, REV=1, BGR=1, TB=1
  LCD_Write_COM_DATA($01,$2B,$3F);

  //Restore VSYNC mode from low power state
  LCD_Write_COM_DATA($02,$06,$00);

  //Sleep mode cancel
  LCD_Write_COM_DATA($10,$00,$00);

  //Entry Mode
  // DFM   0x4000 = 262L color
  // DFM   0x6000 = 65K color
  // AM    0x0000 = horizontal display
  // AM    0x0008 = Vertical display
  // ID[0] 0x0000 = horizontal decrement
  // ID[0] 0x0010 = horizontal increment
  // ID[1] 0x0000 = Vertical decrement
  // ID[1] 0x0020 = Vertical increment
  LCD_Write_COM_DATA($11,$60,$70);

  // Compare register
  LCD_Write_COM_DATA($05,$00,$00);

  // Compare register
  LCD_Write_COM_DATA($06,$00,$00);

  // Horizontal and Vertical porch are for DOTCLK mode operation
  // Horizontal Porch
  LCD_Write_COM_DATA($16,$EF,$1C);

  // Vertical Porch
  LCD_Write_COM_DATA($17,$00,$03);

  // Display Control
  // D1 0x0000 = display off
  // D1 0x0002 = display on
  // D0 0x0000 = internal display halt
  // D0 0x0001 = internal display operate
  LCD_Write_COM_DATA($07,$02,$33);

  // Frame cycle control
  LCD_Write_COM_DATA($0B,$00,$00);

  // Gate Scan Position
  LCD_Write_COM_DATA($0F,$00,$00);

  // Vertical Scroll Control
  LCD_Write_COM_DATA($41,$00,$00);
  LCD_Write_COM_DATA($42,$00,$00);

  // 1st Screen driving position
  // Start position. 0
  LCD_Write_COM_DATA($48,$00,$00);
  // End position.   319
  LCD_Write_COM_DATA($49,$01,$3F);



  // Source RAM address
  //Horizontal RAM address position start/end setup
  //dec 239
  //dec 0, i.e. horizontal ranges from 0 -> 239
  //POR value is 0xEF00 anyway. This address must be set before RAM write
  LCD_Write_COM_DATA($44,$EF,$00);

  //Vertical RAM address start position setting
  //0x0000 = dec 0
  LCD_Write_COM_DATA($45,$00,$00);

  //Vertical RAM address end position setting (0x013F = dec 319)
  LCD_Write_COM_DATA($46,$01,$3F);


  // 2nd Screen driving position
  // Start position. 0
  LCD_Write_COM_DATA($4A,$00,$00);
  // End position.   0
  LCD_Write_COM_DATA($4B,$00,$00);


  //gamma control
  LCD_Write_COM_DATA($30,$07,$07);
  LCD_Write_COM_DATA($31,$02,$04);
  LCD_Write_COM_DATA($32,$02,$04);
  LCD_Write_COM_DATA($33,$05,$02);
  LCD_Write_COM_DATA($34,$05,$07);
  LCD_Write_COM_DATA($35,$02,$04);
  LCD_Write_COM_DATA($36,$02,$04);
  LCD_Write_COM_DATA($37,$05,$02);
  LCD_Write_COM_DATA($3A,$03,$02);
  LCD_Write_COM_DATA($3B,$03,$02);


  LCD_Write_COM_DATA($23,$00,$00);
  LCD_Write_COM_DATA($24,$00,$00);
  LCD_Write_COM_DATA($25,$80,$00);
  LCD_Write_COM_DATA($4F,$00,$00);
  LCD_Write_COM_DATA($4E,$00,$00);
  LCD_Write_COM($22);


  // Chip-Select Low-Aktiv
  LCD_CS_inactive;

end; { procedure Init_SSD1289 }





procedure LCD_Set_Address_Landscape(x1,y1,x2,y2 : Word);
var
  b1 : Byte;
  b2 : Byte;
  w1 : Word;
begin

  w1:=x1;
  x1:=y1;
  y1:=w1;

  w1:=x2;
  x2:=y2;
  y2:=w1;

  y1:=319-y1;
  y2:=319-y2;

  w1:=y1;
  y1:=y2;
  y2:=w1;

  b1:=x1 and $FF;
  b2:=x2 and $FF;
  Lcd_Write_Com_Data($44,b2,b1);

  b1:=y1 and $FF;
  b2:=y1 shr 8;
  Lcd_Write_Com_Data($45,b2,b1);

  b1:=y2 and $FF;
  b2:=y2 shr 8;
  Lcd_Write_Com_Data($46,b2,b1);

  b1:=x1 and $FF;
  b2:=x1 shr 8;
  Lcd_Write_Com_Data($4E,b2,b1);

  b1:=y1 and $FF;
  b2:=y1 shr 8;
  Lcd_Write_Com_Data($4F,b2,b1);

  Lcd_Write_Com($22);

end; { procedure LCD_Set_Address_Landscape }





procedure LCD_Set_Address_Portrait(x1,y1,x2,y2 : Word);
var
  b1 : Byte;
  b2 : Byte;
begin

  b1:=x1 and $FF;
  b2:=x2 and $FF;
  Lcd_Write_Com_Data($44,b2,b1);

  b1:=y1 and $FF;
  b2:=y1 shr 8;
  Lcd_Write_Com_Data($45,b2,b1);

  b1:=y2 and $FF;
  b2:=y2 shr 8;
  Lcd_Write_Com_Data($46,b2,b1);

  b1:=x1 and $FF;
  b2:=x1 shr 8;
  Lcd_Write_Com_Data($4E,b2,b1);

  b1:=y1 and $FF;
  b2:=y1 shr 8;
  Lcd_Write_Com_Data($4F,b2,b1);

  Lcd_Write_Com($22);

end; { procedure LCD_Set_Address_Portrait }







procedure clear_Screen(Pixel_Color : Word);
var
  b1, b2 : Word;
begin

  // Chip-Select Low-Aktiv
  LCD_CS_active;

  LCD_Set_Address_Landscape(0,0,319,239);

  // Data/Command
  // LCD_RS = 1 -> Data
  LCD_RS_Data;

  Write_DATA_Word(Pixel_Color);

  // Write Low-Aktiv
  LCD_WR_active;

  for b1:=1 to 320 do begin

    for b2:=1 to 240 do begin

      NanosecondDelay(5);

      // Write Low-Aktiv
      LCD_WR_inactive;

      NanosecondDelay(10);

      // Write Low-Aktiv
      LCD_WR_active;

    end; { for b2:=1 to 240 do }

  end; { for b1:=1 to 320 do }

  NanosecondDelay(10);

  // Write Low-Aktiv
  LCD_WR_inactive;

  NanosecondDelay(10);

  // Chip-Select Low-Aktiv
  LCD_CS_inactive;

end; { procedure clear_Screen }






procedure drawPixel_Landscape(Pixel_Color, Pos_x, Pos_y : Word);
begin

  // Chip-Select Low-Aktiv
  LCD_CS_active;

  NanosecondDelay(10);

  LCD_Set_Address_Landscape(Pos_x,Pos_y,Pos_x,Pos_y);

  // Data/Command
  // LCD_RS = 1 -> Data
  LCD_RS_Data;

  NanosecondDelay(10);

  Write_Data_Word(Pixel_Color);

  NanosecondDelay(10);

  LCD_WR_active;
  NanosecondDelay(10);
  LCD_WR_inactive;

  NanosecondDelay(10);

  // Chip-Select Low-Aktiv
  LCD_CS_inactive;

end; { procedure drawPixel_Landscape }











procedure drawVertikalLine_Landscape(Pixel_Color, Pos_x, Pos_y1, Pos_y2 : Word);
var
  Line : Word;
begin

  // Chip-Select Low-Aktiv
  LCD_CS_active;

  NanosecondDelay(10);

  LCD_Set_Address_Landscape(Pos_x,Pos_y1,Pos_x,Pos_y2);

  // Data/Command
  // LCD_RS = 1 -> Data
  LCD_RS_Data;

  NanosecondDelay(10);

  Write_Data_Word(Pixel_Color);

  NanosecondDelay(10);

  // Write Low-Aktiv
  LCD_WR_active;

  for Line:=Pos_y1 to Pos_y2 do begin

    NanosecondDelay(5);

    // Write Low-Aktiv
    LCD_WR_inactive;

    NanosecondDelay(10);

    // Write Low-Aktiv
    LCD_WR_active;

  end;

  NanosecondDelay(10);

  // Write Low-Aktiv
  LCD_WR_inactive;

  NanosecondDelay(10);

  // Chip-Select Low-Aktiv
  LCD_CS_inactive;

end; { procedure drawVertikalLine_Landscape }











procedure draw_Picrure(dPicture : Array of Word);
var
  dP_Width, dP_Height : Word;
  dp_Pointer, Array_End : DWord;
  ColorValue1, ColorValue2 : Word;
begin

  dP_Width:=dPicture[0];
  dP_Height:=dPicture[1];

  LCD_CS_active;

  LCD_Set_Address_Portrait(0,0,dP_Width-1,dP_Height-1);

  LCD_RS_Data;

  Array_End:=(dP_Width*dP_Height)+1;

  ColorValue1:=dPicture[Array_End];
  dec(Array_End);

  // LCD_WR_inactive;
  GPIO23_Set;
  Write_DATA_Word(ColorValue1);
  // LCD_WR_active;
  GPIO23_Clear;

  for dp_Pointer:=Array_End downto 2 do begin

    ColorValue2:=ColorValue1;
    ColorValue1:=dPicture[dp_Pointer];

    // LCD_WR_inactive;
    GPIO23_Set;

    if ColorValue1<>ColorValue2 then begin
      Write_DATA_Word(ColorValue1);
    end;

    // LCD_WR_active;
    GPIO23_Clear;

  end; { for Zeiger:=Array_End to 2 do }

  LCD_WR_inactive;
  LCD_CS_inactive;

end; { procedure draw_Picrure }



















{==============================================================================}
{==============================================================================}

initialization

  Init_Ground;
  Init_SSD1289;



{==============================================================================}

finalization
 {Nothing}

{==============================================================================}
{==============================================================================}

end.









end.

